==> datavan-4.2.1.tgz: package/lib/es6/collection-util.js <==
import _mapValues from "lodash/mapValues";
import _groupBy from "lodash/groupBy";
import _keyBy from "lodash/keyBy";
import _each from "lodash/each";
import _filter from "lodash/filter";
import _values from "lodash/values";
import _pickBy from "lodash/pickBy";
import _pick from "lodash/pick";
import _omitBy from "lodash/omitBy";
import _startsWith from "lodash/startsWith";
import Mingo from 'mingo';
import mutateUtil from 'immutability-helper';
export var TMP_ID_PREFIX = 'dv~';
export var tmpIdRegExp = /^dv~(\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d+Z)~([.\d]+)~(.+)/;
export var getDeviceName = function getDeviceName(db) {
  return db && db.deviceName || 'tmp';
}; // NOTE query key with $$ prefix are omitted in query but send to fetcher

var startsWith$$ = function startsWith$$(v, k) {
  return _startsWith(k, '$$');
};

export var genTmpId = function genTmpId(deviceName) {
  return "".concat(TMP_ID_PREFIX).concat(new Date().toISOString(), "~").concat(Math.random(), "~").concat(deviceName || '');
};
export var mingoQuery = function mingoQuery(query) {
  return new Mingo.Query(_omitBy(query, startsWith$$));
};
export var mingoTester = function mingoTester(query) {
  var mQuery = mingoQuery(query);
  return function (doc) {
    return doc && mQuery.test(doc);
  };
};
export var pickBy = function pickBy(byId, query) {
  if (typeof query === 'string' || Array.isArray(query)) return _pick(byId, query);
  if (!query) return byId;
  return _pickBy(byId, mingoTester(query));
};
export var filter = function filter(byId, query) {
  if (!query) return _values(byId);
  return _filter(byId, mingoTester(query));
};
mutateUtil.extend('$auto', function (value, object) {
  return object ? mutateUtil(object, value) : mutateUtil({}, value);
});
export { mutateUtil }; // @auto-fold here prepare cast by loop mutation.byId and mark [id] $toggle $merge

var checkCast = function checkCast(coll, nextById, prevById, id) {
  var nextDoc = nextById[id];

  if (nextDoc !== prevById[id] && nextDoc) {
    nextById[id] = coll.cast(nextDoc) || nextDoc;
  }
}; // @auto-fold


export var checkCastById = function checkCastById(space, next, prev, mutation) {
  var nextById = next[space];
  var prevById = prev[space];
  if (nextById === prevById) return;

  _each(mutation[space], function (v, id) {
    if (id[0] === '$') {
      if (id === '$merge' || id === '$toggle') {
        _each(v, function (subV, subId) {
          return checkCast(next, nextById, prevById, subId);
        });
      }
    } else {
      checkCast(next, nextById, prevById, id);
    }
  });
};
export var buildIndex = function buildIndex(docs, fields, isUnique) {
  fields = Array.isArray(fields) ? fields : [fields];
  var field = fields[0];

  if (fields.length === 1) {
    return isUnique ? _keyBy(docs, field) : _groupBy(docs, field);
  }

  var restSteps = fields.slice(1);

  var groups = _groupBy(docs, field);

  return _mapValues(groups, function (groupDocs) {
    return buildIndex(groupDocs, restSteps, isUnique);
  });
};

==> datavan-4.2.5.tgz: package/lib/es6/collection-util.js <==
import _mapValues from "lodash/mapValues";
import _groupBy from "lodash/groupBy";
import _keyBy from "lodash/keyBy";
import _each from "lodash/each";
import _transform from "lodash/transform";
import _keys from "lodash/keys";
import _filter from "lodash/filter";
import _values from "lodash/values";
import _pickBy from "lodash/pickBy";
import _pick from "lodash/pick";
import _omitBy from "lodash/omitBy";
import _startsWith from "lodash/startsWith";
import Mingo from 'mingo';
import mutateUtil from 'immutability-helper';
export var TMP_ID_PREFIX = 'dv~';
export var tmpIdRegExp = /^dv~(\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d+Z)~([.\d]+)~(.+)/;
export var getDeviceName = function getDeviceName(db) {
  return db && db.deviceName || 'tmp';
}; // NOTE query key with $$ prefix are omitted in query but send to fetcher

var startsWith$$ = function startsWith$$(v, k) {
  return _startsWith(k, '$$');
};

export var genTmpId = function genTmpId(deviceName) {
  return "".concat(TMP_ID_PREFIX).concat(new Date().toISOString(), "~").concat(Math.random(), "~").concat(deviceName || '');
};
export var mingoQuery = function mingoQuery(query) {
  return new Mingo.Query(_omitBy(query, startsWith$$));
};
export var mingoTester = function mingoTester(query) {
  var mQuery = mingoQuery(query);
  return function (doc) {
    return doc && mQuery.test(doc);
  };
};
export var pickBy = function pickBy(byId, query) {
  if (typeof query === 'string' || Array.isArray(query)) return _pick(byId, query);
  if (!query) return byId;
  return _pickBy(byId, mingoTester(query));
};
export var filter = function filter(byId, query) {
  if (!query) return _values(byId);
  return _filter(byId, mingoTester(query));
};
mutateUtil.extend('$auto', function (value, object) {
  return object ? mutateUtil(object, value) : mutateUtil({}, value);
});
export { mutateUtil }; // @auto-fold

export var flattenMutationKeys = function flattenMutationKeys(mutation, skipUnset) {
  return _keys(_transform(mutation, function (ret, value, key) {
    if (key[0] === '$') {
      if (!skipUnset && key === '$unset') {
        _each(value, function (k) {
          ret[k] = true;
        });
      } else if (key === '$merge' || key === '$toggle') {
        _each(value, function (v, k) {
          ret[k] = true;
        });
      }
    } else {
      ret[key] = true;
    }
  }));
}; // @auto-fold here prepare cast by loop mutation.byId and mark [id] $toggle $merge

var checkCast = function checkCast(coll, nextById, prevById, id) {
  var nextDoc = nextById[id];

  if (nextDoc !== prevById[id] && nextDoc) {
    nextById[id] = coll.cast(nextDoc) || nextDoc;
  }
}; // @auto-fold


export var checkCastById = function checkCastById(space, next, prev, mutation) {
  var nextById = next[space];
  var prevById = prev[space];
  if (nextById === prevById) return;
  flattenMutationKeys(mutation[space], true).forEach(function (id) {
    checkCast(next, nextById, prevById, id);
  });
};
export var buildIndex = function buildIndex(docs, fields, isUnique) {
  fields = Array.isArray(fields) ? fields : [fields];
  var field = fields[0];

  if (fields.length === 1) {
    return isUnique ? _keyBy(docs, field) : _groupBy(docs, field);
  }

  var restSteps = fields.slice(1);

  var groups = _groupBy(docs, field);

  return _mapValues(groups, function (groupDocs) {
    return buildIndex(groupDocs, restSteps, isUnique);
  });
};
